#include<stdio.h>
#include<math.h>
#include "TNR_VECTOR.hxx"

#define EPS 1.0e-5
#define LIMIT 10000

class CURVE{
   TNR_POINT p[4];
public:
	CURVE(){
		p[0] = TNR_POINT(0.0, 0.0, 0.0);
		p[1] = TNR_POINT(1.0, 1.0, 0.0);
		p[2] = TNR_POINT(2.0, 0.0, 0.0);
		p[3] = TNR_POINT(3.0, 1.0, 0.0);

	}
	/* get position on curve */
	TNR_POINT getPosition(double u){
		double k0 =  0.5*(1.0-u)*(1.0-3.0*u)*(2.0-3.0*u);
		double k1 =  4.5*u*(1.0-u)*(2.0-3.0*u);
		double k2 = -4.5*u*(1.0-u)*(1.0-3.0*u);
		double k3 =  0.5*u*(1.0-3.0*u)*(2.0-3.0*u);
		return k0*p[0] + k1*p[1] + k2*p[2] + k3*p[3];
	}

	/* get tangent vectgor */
	TNR_VECTOR getTangentVector(double u){
		double k0 =  0.5*( -11 + 36 * u - 27 * u * u );
		double k1 =  4.5*( 2.0 - 10 * u + 9 * u * u );
		double k2 = -4.5*( 1.0 - 8 * u + 9 * u * u);
		double k3 =  0.5*( 2.0 - 18 * u + 27 * u * u);
                   return k0*p[0] + k1*p[1] + k2*p[2] + k3*p[3];
	}

};


double clippedOfParameter(double u)
{
	if(u < 0.0){
       return 0.0;
	}
	if(u > 1.0){
       return 1.0;
	}
	return u;
}

int main()
{
   CURVE* curve = new CURVE;

   TNR_POINT r(1.5,1.0,0.0);
   double  u = 0.3;

   for(int i = 0;i <=LIMIT;i++){
     //tangent vector
     TNR_VECTOR tangentVector =  curve->getTangentVector(u);
     // norm of tangent vector
 double norm = tangentVector.norm();
 // unit tangent vector
     TNR_VECTOR unitTangentVector = tangentVector.normalize();
TNR_VECTOR r_Ru = r - curve->getPosition(u);
 double dist = dotProduct(unitTangentVector,r_Ru);
 //The distance is minimized when tangent vector and "r - R(u)" are vertical. 
 if(fabs(dist) < EPS){
  printf("%d) end calculate.\n",i);
       printf("dist=%f\n",fabs(dist));
   printf("dist=%f  parameter is  =%f\n",distanceBetween(r,curve->getPosition(u)),u);
return (0);
 }
 //set new u parameter
u = u + (dist / norm);

   }
   printf("out of limit(%f)!!\n",LIMIT);
   return 0;
}
