#include<stdio.h>
#include<math.h>
#include<float.h>

#include "TNR_VECTOR.hxx"

#define EPS 1.0e-5
#define LIMIT 10000



class CURVE{
   TNR_POINT p[4];
public:
	CURVE(){
		p[0] = TNR_POINT(0.0, 0.0, 0.0);
		p[1] = TNR_POINT(1.0, 1.0, 0.0);
		p[2] = TNR_POINT(2.0, 0.0, 0.0);
		p[3] = TNR_POINT(3.0, 1.0, 0.0);

	}
	/* get position on curve */
	TNR_POINT getPosition(double u){
		double k0 =  0.5*(1.0-u)*(1.0-3.0*u)*(2.0-3.0*u);
		double k1 =  4.5*u*(1.0-u)*(2.0-3.0*u);
		double k2 = -4.5*u*(1.0-u)*(1.0-3.0*u);
		double k3 =  0.5*u*(1.0-3.0*u)*(2.0-3.0*u);
		return k0*p[0] + k1*p[1] + k2*p[2] + k3*p[3];
	}

	/* get tangent vectgor */
	TNR_VECTOR getTangentVector(double u){
		double k0 =  0.5*( -11 + 36 * u - 27 * u * u );
		double k1 =  4.5*( 2.0 - 10 * u + 9 * u * u );
		double k2 = -4.5*( 1.0 - 8 * u + 9 * u * u);
		double k3 =  0.5*( 2.0 - 18 * u + 27 * u * u);
        return k0*p[0] + k1*p[1] + k2*p[2] + k3*p[3];
	}
};


//get  distance between target point and point on curve(param).
// f(u) =| R(u) - r |^2
double f(CURVE* curve,double u,TNR_POINT r)
{
  TNR_VECTOR t = r - curve->getPosition(u);
  return dotProduct(t,t) ; 
}

// f(u)' =d(u)

double d(CURVE* curve,double u,TNR_POINT r)
{
  TNR_VECTOR fu = curve->getTangentVector(u).normalize();
  TNR_VECTOR vec = r - curve->getPosition(u);
  return dotProduct(fu,vec);
}

double clippedOfParameter(double u)
{
	if(u < 0.0){
       return 0.0;
	}
	if(u > 1.0){
       return 1.0;
	}
	return u;
}



void initialGuessforBisection(CURVE* curve,TNR_POINT r,double* retLowValue,double* retHightValue)
{
  int accuracy = 10; //number of sampling points is accuracy + 1.
  double u = 0.0;    
  double incre = 1.0/accuracy; //increment of u
  double minDist = DBL_MAX;    //minimum of distance about sampling points
  double nearU = 0.0;          //near point
  for(int i = 0; i < accuracy + 1; i++,u += incre){
    TNR_POINT pos = curve->getPosition(u);
	TNR_VECTOR d = r -pos;
    double dist = dotProduct(d,d);
    if(dist < minDist){
      minDist = dist;
	  nearU = u;
    }
  }

  *retLowValue   = clippedOfParameter(nearU - incre);
  *retHightValue = clippedOfParameter(nearU + incre);
  
}

int main()
{

   CURVE* curve = new CURVE; 
   TNR_POINT r(1.5,1.0,0.0); //projected point


   //calculate range (low and hight) of initial guess 
   double low;
   double hight;
   initialGuessforBisection(curve,r,&low,&hight);


   for(int i = 0;i <=LIMIT;i++){
	  double u = (low+hight) / 2;
	  double t = d(curve,u,r); 

	  if(t > 0.0){
        low = u;
	  }else{
	    hight= u;
	  }

      double fu = fabs(d(curve,u,r));
	  double uDiff = fabs(( hight - low ) /low);

	  printf("low = %f, hight = %f\n",low,hight);

	  if(  fu < EPS ) {
	    printf("%d) end calculate.\n",i);
		printf("param u = %f\n",u);
	    return (0);
	  }

   }
   printf("out of limit(%f)!!\n",LIMIT);

   return 0;
}


